@@ -49,10 +49,6 @@ static char rcsid[] = "$Id: edquota.c,v 1.6 1995/07/23 09:58:06 mvw Exp mvw $";
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/file.h>
-#define __KERNEL__
-#include <linux/kdev_t.h>
-#include <linux/wait.h>
-#undef __KERNEL__
 #include <linux/quota.h>
 #include <errno.h>
 #include <mntent.h>
@@ -62,6 +58,7 @@ static char rcsid[] = "$Id: edquota.c,v 1.6 1995/07/23 09:58:06 mvw Exp mvw $";
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
+#include <unistd.h>
 #include "pathnames.h"
 
 extern char *qfextension[];
@@ -78,9 +75,9 @@ struct quotause {
 
 #define   FOUND   0x01
 
-main(int  argc, char **argv)
+main(int argc, char **argv)
 {
-   struct quotause *qup, *protoprivs, *curprivs;
+   struct quotause *qup, *protoprivs, *curprivs, *pprivs, *cprivs;
    extern char *optarg;
    extern int optind;
    long id, protoid;
@@ -90,10 +87,7 @@ main(int  argc, char **argv)
 
    if (argc < 2)
       usage();
-   if (getuid()) {
-      fprintf(stderr, "edquota: permission denied\n");
-      exit(1);
-   }
+
    quotatype = USRQUOTA;
    while ((ch = getopt(argc, argv, "ugtp:")) != EOF) {
       switch (ch) {
@@ -116,6 +110,7 @@ main(int  argc, char **argv)
    }
    argc -= optind;
    argv += optind;
+
    if (pflag) {
       if ((protoid = getentry(protoname, quotatype)) == -1)
          exit(1);
@@ -128,12 +123,23 @@ main(int  argc, char **argv)
          if ((id = getentry(*argv++, quotatype)) < 0)
             continue;
          curprivs = getprivs(id, quotatype);
-         protoprivs->dqblk.dqb_curblocks = curprivs->dqblk.dqb_curblocks;
-         protoprivs->dqblk.dqb_curinodes = curprivs->dqblk.dqb_curinodes;
+
+	 for (pprivs = protoprivs, cprivs = curprivs;
+	      pprivs && cprivs;
+	      pprivs = pprivs->next, cprivs = cprivs->next) {
+	    if (strcmp(pprivs->fsname, cprivs->fsname)) {
+	       fprintf(stderr, "fsname mismatch\n");
+	    } else {
+	       pprivs->dqblk.dqb_curblocks = cprivs->dqblk.dqb_curblocks;
+	       pprivs->dqblk.dqb_curinodes = cprivs->dqblk.dqb_curinodes;
+	    }
+	 }
          putprivs(id, quotatype, protoprivs);
       }
       exit(0);
    }
+
+   umask(077);
    tmpfd = mkstemp(tmpfil);
    fchown(tmpfd, getuid(), getgid());
    if (tflag) {
@@ -143,18 +149,21 @@ main(int  argc, char **argv)
       if (editit(tmpfil) && readtimes(protoprivs, tmpfd))
          putprivs(0, quotatype, protoprivs);
       freeprivs(protoprivs);
-      exit(0);
-   }
-   for (; argc > 0; argc--, argv++) {
-      if ((id = getentry(*argv, quotatype)) == -1)
-         continue;
-      curprivs = getprivs(id, quotatype);
-      if (writeprivs(curprivs, tmpfd, *argv, quotatype) == 0)
-         continue;
-      if (editit(tmpfil) && readprivs(curprivs, tmpfd))
-         putprivs(id, quotatype, curprivs);
-      freeprivs(curprivs);
+      close(tmpfd);
+      unlink(tmpfil);
+   } else {
+      for (; argc > 0; argc--, argv++) {
+         if ((id = getentry(*argv, quotatype)) == -1)
+            continue;
+         curprivs = getprivs(id, quotatype);
+         if (writeprivs(curprivs, tmpfd, *argv, quotatype) == 0)
+            continue;
+         if (editit(tmpfil) && readprivs(curprivs, tmpfd))
+            putprivs(id, quotatype, curprivs);
+         freeprivs(curprivs);
+      }
    }
+
    close(tmpfd);
    unlink(tmpfil);
    exit(0);
@@ -163,7 +172,7 @@ main(int  argc, char **argv)
 usage(void)
 {
    fprintf(stderr, "%s%s%s%s",
-      "Usage: edquota [-u] [-p username] username ...\n",
+      "Usage:\tedquota [-u] [-p username] username ...\n",
       "\tedquota -g [-p groupname] groupname ...\n",
       "\tedquota [-u] -t\n", "\tedquota -g -t\n");
    exit(1);
@@ -218,13 +227,20 @@ struct quotause *getprivs(long id, int quotatype)
    while ((mnt = getmntent(fp)) != (struct mntent *) 0) {
       if (hasmntopt(mnt, MNTOPT_NOAUTO) || !hasquota(mnt, quotatype, &qfpathname))
          continue;
+
+      if (access(qfpathname, R_OK | W_OK)) {
+         fprintf(stderr, "edquota: open %s permission denied\n", qfpathname);
+         continue;
+      }
+
       qupsize = sizeof(*qup) + strlen(qfpathname);
       if ((qup = (struct quotause *) malloc(qupsize)) == (struct quotause *)NULL) {
          fprintf(stderr, "edquota: out of memory\n");
          exit(2);
       }
+
       if (quotactl(qcmd, mnt->mnt_fsname, id, (caddr_t) &qup->dqblk) != 0) {
-         if ((errno == EOPNOTSUPP || errno == ENOSYS) && !warned) {
+         if ((errno == EOPNOTSUPP || errno == ENOSYS || errno == ENOPKG) && !warned) {
             warned++;
             fprintf(stderr, "Warning: %s\n",
             "Quotas are not compiled into this kernel");
@@ -243,6 +259,7 @@ struct quotause *getprivs(long id, int quotatype)
                    getentry(quotagroup, GRPQUOTA));
             (void) fchmod(fd, 0640);
          }
+
          lseek(fd, (long) (id * sizeof(struct dqblk)), L_SET);
          switch (read(fd, &qup->dqblk, sizeof(struct dqblk))) {
             case 0:/* EOF */
@@ -253,10 +270,8 @@ struct quotause *getprivs(long id, int quotatype)
                bzero((caddr_t) & qup->dqblk,
                      sizeof(struct dqblk));
                break;
-
             case sizeof(struct dqblk):   /* OK */
                break;
-
             default:   /* ERROR */
                fprintf(stderr, "edquota: read error in ");
                perror(qfpathname);
@@ -266,6 +281,7 @@ struct quotause *getprivs(long id, int quotatype)
          }
          close(fd);
       }
+
       strcpy(qup->qfname, qfpathname);
       strcpy(qup->fsname, mnt->mnt_fsname);
       if (quphead == NULL)
@@ -327,8 +343,9 @@ editit(char *tmpfile)
       sigsetmask(omask);
       setgid(getgid());
       setuid(getuid());
-      if ((ed = getenv("EDITOR")) == (char *) 0)
-         ed = "/bin/ae";
+      if ((ed = getenv("VISUAL")) == (char *) 0)
+         if ((ed = getenv("EDITOR")) == (char *) 0)
+            ed = _PATH_VI;
       execlp(ed, ed, tmpfile, 0);
       perror(ed);
       exit(1);
@@ -354,17 +371,37 @@ writeprivs(struct quotause *quplist, int outfd, char *name, int quotatype)
       perror(tmpfil);
       exit(1);
    }
+
+#if defined(ALT_FORMAT)
+   fprintf(fd, "Disk quotas for %s %s (%cid %d):\n", 
+           qfextension[quotatype], name,
+           *qfextension[quotatype], getentry(name, quotatype));
+
+   fprintf(fd, "  Filesystem       blocks       soft       hard     inodes     soft     hard\n");
+
+   for (qup = quplist; qup; qup = qup->next) {
+      fprintf(fd, "  %-12s %10d %10d %10d %10d %8d %8d\n",
+              qup->fsname,
+              dbtob(qup->dqblk.dqb_curblocks) / 1024,
+              dbtob(qup->dqblk.dqb_bsoftlimit) / 1024,
+              dbtob(qup->dqblk.dqb_bhardlimit) / 1024,
+              qup->dqblk.dqb_curinodes,
+              qup->dqblk.dqb_isoftlimit, 
+	      qup->dqblk.dqb_ihardlimit);
+   }
+#else
    fprintf(fd, "Quotas for %s %s:\n", qfextension[quotatype], name);
    for (qup = quplist; qup; qup = qup->next) {
       fprintf(fd, "%s: %s %d, limits (soft = %d, hard = %d)\n",
-         qup->fsname, "blocks in use:",
-         dbtob(qup->dqblk.dqb_curblocks) / 1024,
-         dbtob(qup->dqblk.dqb_bsoftlimit) / 1024,
-         dbtob(qup->dqblk.dqb_bhardlimit) / 1024);
+              qup->fsname, "blocks in use:",
+              dbtob(qup->dqblk.dqb_curblocks) / 1024,
+              dbtob(qup->dqblk.dqb_bsoftlimit) / 1024,
+              dbtob(qup->dqblk.dqb_bhardlimit) / 1024);
       fprintf(fd, "%s %d, limits (soft = %d, hard = %d)\n",
-         "\tinodes in use:", qup->dqblk.dqb_curinodes,
-            qup->dqblk.dqb_isoftlimit, qup->dqblk.dqb_ihardlimit);
+              "\tinodes in use:", qup->dqblk.dqb_curinodes,
+              qup->dqblk.dqb_isoftlimit, qup->dqblk.dqb_ihardlimit);
    }
+#endif
    fclose(fd);
    return (1);
 }
@@ -379,7 +416,11 @@ readprivs(struct quotause *quplist, int infd)
    int cnt;
    char *cp;
    struct dqblk dqblk;
+#if defined(ALT_FORMAT)
+   char fsp[BUFSIZ], line[BUFSIZ];
+#else
    char *fsp, line1[BUFSIZ], line2[BUFSIZ];
+#endif
 
    lseek(infd, 0, L_SET);
    fd = fdopen(dup(infd), "r");
@@ -387,6 +428,63 @@ readprivs(struct quotause *quplist, int infd)
       fprintf(stderr, "Can't re-read temp file!!\n");
       return (0);
    }
+
+#if defined(ALT_FORMAT)
+   /*
+    * Discard title lines, then read lines to process.
+    */
+   (void) fgets(line, sizeof(line), fd);
+   (void) fgets(line, sizeof(line), fd);
+	
+   while (fgets(line, sizeof(line), fd)) {
+      cnt = sscanf(line, "%s %d %d %d %d %d %d",
+	           fsp, &dqblk.dqb_curblocks, &dqblk.dqb_bsoftlimit,
+                   &dqblk.dqb_bhardlimit, &dqblk.dqb_curinodes,
+                   &dqblk.dqb_isoftlimit, &dqblk.dqb_ihardlimit);
+
+      if (cnt != 7) {
+         fprintf(stderr, "bad format:\n%s\n", line);
+         return (0);
+      }
+	   
+      dqblk.dqb_curblocks  = btodb(dqblk.dqb_curblocks * 1024);
+      dqblk.dqb_bsoftlimit = btodb(dqblk.dqb_bsoftlimit * 1024);
+      dqblk.dqb_bhardlimit = btodb(dqblk.dqb_bhardlimit * 1024);
+
+      for (qup = quplist; qup; qup = qup->next) {
+         if (strcmp(fsp, qup->fsname))
+            continue;
+         /*
+          * Cause time limit to be reset when the quota is
+          * next used if previously had no soft limit or were
+          * under it, but now have a soft limit and are over
+          * it.
+          */
+         if (dqblk.dqb_bsoftlimit &&
+            (qup->dqblk.dqb_curblocks >= dqblk.dqb_bsoftlimit) &&
+            (qup->dqblk.dqb_bsoftlimit == 0 ||
+             qup->dqblk.dqb_curblocks < qup->dqblk.dqb_bsoftlimit))
+            qup->dqblk.dqb_btime = 0;
+
+         if (dqblk.dqb_isoftlimit &&
+            (qup->dqblk.dqb_curinodes >= dqblk.dqb_isoftlimit) &&
+            (qup->dqblk.dqb_isoftlimit == 0 ||
+             qup->dqblk.dqb_curinodes < qup->dqblk.dqb_isoftlimit))
+            qup->dqblk.dqb_itime = 0;
+
+         qup->dqblk.dqb_bsoftlimit = dqblk.dqb_bsoftlimit;
+         qup->dqblk.dqb_bhardlimit = dqblk.dqb_bhardlimit;
+         qup->dqblk.dqb_isoftlimit = dqblk.dqb_isoftlimit;
+         qup->dqblk.dqb_ihardlimit = dqblk.dqb_ihardlimit;
+         qup->flags |= FOUND;
+
+         if (dqblk.dqb_curblocks != qup->dqblk.dqb_curblocks)
+            fprintf(stderr, "%s: cannot change current block allocation\n", fsp);
+	 if (dqblk.dqb_curinodes != qup->dqblk.dqb_curinodes)
+            fprintf(stderr, "%s: cannot change current inode allocation\n", fsp);
+      }
+   }
+#else
    /*
     * Discard title line, then read pairs of lines to process.
     */
@@ -402,29 +500,30 @@ readprivs(struct quotause *quplist, int infd)
             &fsp[strlen(fsp) + 1]);
          return (0);
       }
-      cnt = sscanf(cp,
-         " blocks in use: %d, limits (soft = %d, hard = %d)",
-              &dqblk.dqb_curblocks, &dqblk.dqb_bsoftlimit,
-              &dqblk.dqb_bhardlimit);
+
+      cnt = sscanf(cp, " blocks in use: %d, limits (soft = %d, hard = %d)",
+                   &dqblk.dqb_curblocks, &dqblk.dqb_bsoftlimit, &dqblk.dqb_bhardlimit);
       if (cnt != 3) {
          fprintf(stderr, "%s:%s: bad format\n", fsp, cp);
          return (0);
       }
+
       dqblk.dqb_curblocks = btodb(dqblk.dqb_curblocks * 1024);
       dqblk.dqb_bsoftlimit = btodb(dqblk.dqb_bsoftlimit * 1024);
       dqblk.dqb_bhardlimit = btodb(dqblk.dqb_bhardlimit * 1024);
+
       if ((cp = strtok(line2, "\n")) == NULL) {
          fprintf(stderr, "%s: %s: bad format\n", fsp, line2);
          return (0);
       }
-      cnt = sscanf(cp,
-             "\tinodes in use: %d, limits (soft = %d, hard = %d)",
-              &dqblk.dqb_curinodes, &dqblk.dqb_isoftlimit,
-              &dqblk.dqb_ihardlimit);
+
+      cnt = sscanf(cp, "\tinodes in use: %d, limits (soft = %d, hard = %d)",
+                   &dqblk.dqb_curinodes, &dqblk.dqb_isoftlimit, &dqblk.dqb_ihardlimit);
       if (cnt != 3) {
          fprintf(stderr, "%s: %s: bad format\n", fsp, line2);
          return (0);
       }
+
       for (qup = quplist; qup; qup = qup->next) {
          if (strcmp(fsp, qup->fsname))
             continue;
@@ -435,17 +534,19 @@ readprivs(struct quotause *quplist, int infd)
           * it.
           */
          if (dqblk.dqb_bsoftlimit &&
-          qup->dqblk.dqb_curblocks >= dqblk.dqb_bsoftlimit &&
+             qup->dqblk.dqb_curblocks >= dqblk.dqb_bsoftlimit &&
              (qup->dqblk.dqb_bsoftlimit == 0 ||
               qup->dqblk.dqb_curblocks <
               qup->dqblk.dqb_bsoftlimit))
             qup->dqblk.dqb_btime = 0;
+
          if (dqblk.dqb_isoftlimit &&
-          qup->dqblk.dqb_curinodes >= dqblk.dqb_isoftlimit &&
+             qup->dqblk.dqb_curinodes >= dqblk.dqb_isoftlimit &&
              (qup->dqblk.dqb_isoftlimit == 0 ||
               qup->dqblk.dqb_curinodes <
               qup->dqblk.dqb_isoftlimit))
             qup->dqblk.dqb_itime = 0;
+
          qup->dqblk.dqb_bsoftlimit = dqblk.dqb_bsoftlimit;
          qup->dqblk.dqb_bhardlimit = dqblk.dqb_bhardlimit;
          qup->dqblk.dqb_isoftlimit = dqblk.dqb_isoftlimit;
@@ -460,7 +561,9 @@ readprivs(struct quotause *quplist, int infd)
          break;
       }
    }
+#endif
    fclose(fd);
+
    /*
     * Disable quotas for any filesystems that have not been found.
     */
@@ -493,6 +596,17 @@ writetimes(struct quotause *quplist, int outfd, int quotatype)
       perror(tmpfil);
       exit(1);
    }
+
+#if defined(ALT_FORMAT)
+   fprintf(fd, "Grace period before enforcing soft limits for %ss:\n", qfextension[quotatype]);
+   fprintf(fd, "Time units may be: days, hours, minutes, or seconds\n");
+   fprintf(fd, "  Filesystem             Block grace period     Inode grace period\n");
+
+   for (qup = quplist; qup; qup = qup->next) {
+      fprintf(fd, "  %-12s %22s ", qup->fsname, cvtstoa(qup->dqblk.dqb_btime));
+      fprintf(fd, "%22s\n", cvtstoa(qup->dqblk.dqb_itime));
+   }
+#else
    fprintf(fd, "Time units may be: days, hours, minutes, or seconds\n");
    fprintf(fd, "Grace period before enforcing soft limits for %ss:\n",
       qfextension[quotatype]);
@@ -502,6 +616,8 @@ writetimes(struct quotause *quplist, int outfd, int quotatype)
       fprintf(fd, "file grace period: %s\n",
          cvtstoa(qup->dqblk.dqb_itime));
    }
+#endif
+
    fclose(fd);
    return (1);
 }
@@ -516,7 +632,11 @@ readtimes(struct quotause *quplist, int infd)
    int cnt;
    char  *cp;
    time_t itime, btime, iseconds, bseconds;
+#if defined(ALT_FORMAT)
+   char fsp[BUFSIZ], bunits[10], iunits[10], line[BUFSIZ];
+#else
    char *fsp, bunits[10], iunits[10], line1[BUFSIZ];
+#endif
 
    lseek(infd, 0, L_SET);
    fd = fdopen(dup(infd), "r");
@@ -524,11 +644,29 @@ readtimes(struct quotause *quplist, int infd)
       fprintf(stderr, "Can't re-read temp file!!\n");
       return (0);
    }
+
+#if defined(ALT_FORMAT)
+   /*
+    * Discard three title lines, then read lines to process.
+    */
+   (void) fgets(line, sizeof(line), fd);
+   (void) fgets(line, sizeof(line), fd);
+   (void) fgets(line, sizeof(line), fd);
+
+   while (fgets(line, sizeof(line), fd) != NULL) {
+      cnt = sscanf(line, "%s %d %s %d %s",
+                   fsp, &btime, bunits, &itime, iunits);
+      if (cnt != 5) {
+         fprintf(stderr, "bad format:\n%s\n", line);
+         return (0);
+      }
+#else
    /*
     * Discard two title lines, then read lines to process.
     */
    (void) fgets(line1, sizeof(line1), fd);
    (void) fgets(line1, sizeof(line1), fd);
+
    while (fgets(line1, sizeof(line1), fd) != NULL) {
       if ((fsp = strtok(line1, " \t:")) == NULL) {
          fprintf(stderr, "%s: bad format\n", line1);
@@ -546,6 +684,7 @@ readtimes(struct quotause *quplist, int infd)
          fprintf(stderr, "%s:%s: bad format\n", fsp, cp);
          return (0);
       }
+#endif
       if (cvtatos(btime, bunits, &bseconds) == 0)
          return (0);
       if (cvtatos(itime, iunits, &iseconds) == 0)
@@ -560,6 +699,7 @@ readtimes(struct quotause *quplist, int infd)
       }
    }
    fclose(fd);
+
    /*
     * reset default grace periods for any filesystems that have not been
     * found.
@@ -644,4 +784,3 @@ alldigits(char *s)
    } while (c = *s++);
    return (1);
 }
-
