@@ -21,10 +21,6 @@
  */
 #include <sys/types.h>
 #include <limits.h>
-#define __KERNEL__
-#include <linux/kdev_t.h>
-#include <linux/wait.h>
-#undef __KERNEL__
 #include <linux/quota.h>
 #include <string.h>
 #include <mntent.h>
@@ -65,4 +61,3 @@ hasquota(struct mntent *mnt, int type, char **qfnamep)
    } else
       return (0);
 }
-
