@@ -49,10 +49,6 @@ static char rcsid[] = "$Id: quota.c,v 1.3 1995/05/15 13:49:14 mvw Exp $";
 #include <sys/param.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-#define __KERNEL__
-#include <linux/kdev_t.h>
-#include <linux/wait.h>
-#undef __KERNEL__
 #include <linux/quota.h>
 #include <stdio.h>
 #include <mntent.h>
@@ -83,6 +79,28 @@ int vflag;
 int showuid(uid_t uid);
 int showgid(gid_t gid);
 
+void showversion(void)
+{
+  printf("quota "QUOTA_VERSION
+#if defined(RPC) || defined(EXT2_DIRECT)
+  ", with "
+#ifdef RPC
+  "RPC "
+#endif
+#ifdef EXT2_DIRECT
+#ifdef RPC
+  "and "
+#endif
+  "EXT2_DIRECT "
+#endif
+  "options"
+#else
+  ", without special options"
+#endif
+  "\n");
+  exit(1);
+}
+
 main(int argc, char **argv)
 {
    int ngroups;
@@ -92,7 +110,7 @@ main(int argc, char **argv)
    extern char *optarg;
    extern int optind, errno;
 
-   while ((ch = getopt(argc, argv, "ugvq")) != EOF) {
+   while ((ch = getopt(argc, argv, "ugvqV")) != EOF) {
       switch (ch) {
          case 'g':
             gflag++;
@@ -106,6 +124,8 @@ main(int argc, char **argv)
          case 'q':
             qflag++;
             break;
+         case 'V':
+	    showversion();
          default:
             usage();
       }
@@ -130,6 +150,7 @@ main(int argc, char **argv)
       }
       exit(0);
    }
+
    if (uflag && gflag)
       usage();
 
@@ -142,6 +163,7 @@ main(int argc, char **argv)
       }
       exit(0);
    }
+
    if (gflag) {
       for (; argc > 0; argc--, argv++) {
          if (alldigits(*argv))
@@ -156,7 +178,7 @@ main(int argc, char **argv)
 usage(void)
 {
    fprintf(stderr, "%s\n%s\n%s\n",
-      "Usage: quota [-guqv]",
+      "Usage: quota [-guqvV]",
       "\tquota [-qv] -u username ...",
       "\tquota [-qv] -g groupname ...");
    exit(1);
@@ -521,7 +543,7 @@ struct quotause *getprivs(long id, int quotatype)
 
       if (strcmp(mnt->mnt_type, MNTTYPE_NFS)) {
          if (quotactl(qcmd, mnt->mnt_fsname, id, (caddr_t)&qup->dqblk) != 0) {
-            if (errno == ENOSYS || errno == ESRCH)
+            if (errno == ENOSYS || errno == ESRCH )
                continue;
             if ((fd = open(qfpathname, O_RDONLY)) < 0) {
                perror(qfpathname);
