@@ -24,10 +24,6 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <string.h>
-#define __KERNEL__
-#include <linux/kdev_t.h>
-#include <linux/wait.h>
-#undef __KERNEL__
 #include <linux/quota.h>
 #include <stdarg.h>
 #include <sys/file.h>
@@ -50,7 +46,7 @@ static char RCS_checkquota[] = "$Id: quotacheck.c,v 2.12 1995/07/23 09:58:06 mvw
 #endif
 
 struct dquot {
-   int          dq_id;  /* id this applies to (uid, gid) */
+   int dq_id;  /* id this applies to (uid, gid) */
    struct dqblk dq_dqb; /* diskquota for id */
    struct dquot *next;  /* pointer to next id */
 };
@@ -200,7 +196,8 @@ int main(int argc, char **argv)
    int cnt, ch;
    struct stat st;
    char aflag = 0, gflag = 0, uflag = 0;
-   __u32 argnum, done;
+   unsigned done;
+   int argnum;
    char *usr_qfnp, *grp_qfnp;
    register struct mntent *mnt;
 
@@ -242,13 +239,22 @@ int main(int argc, char **argv)
    fp = setmntent(MNTTAB, "r");
    while ((mnt = getmntent(fp)) != (struct mntent *) NULL) {
       check_usr = check_grp = 0;
-      if (argc && ((argnum = oneof(mnt->mnt_dir, argv, argc)) >= 0) ||
-                  ((argnum = oneof(mnt->mnt_fsname, argv, argc)) >= 0)) {
-          done |= 1 << argnum;
-      } else
-         if (!aflag || hasmntopt(mnt, MNTOPT_NOAUTO) ||
-                       hasmntopt(mnt, MNTOPT_NOQUOTA))
-            continue;
+
+      if (!aflag) { /* .. or: argc != 0 */
+	/* If the user specified a fs/part name, then check only
+	   those, and skip all others */
+	if (((argnum = oneof(mnt->mnt_dir, argv, argc)) >= 0) ||
+	    ((argnum = oneof(mnt->mnt_fsname, argv, argc)) >= 0)) {
+	  done |= (1<<argnum);
+	} else
+	  continue;
+      } else {
+	/* If 'all', then skip every fs, which is not marked as 'auto'
+	   AND user or group quota */
+	if (hasmntopt(mnt, MNTOPT_NOAUTO) ||
+	    hasmntopt(mnt, MNTOPT_NOQUOTA))
+	  continue;
+      }
 
       if (gflag && hasquota(mnt, GRPQUOTA, &grp_qfnp))
          check_grp++;
@@ -277,7 +283,7 @@ int main(int argc, char **argv)
 #if defined(EXT2_DIRECT)
            if (!strcmp(mnt->mnt_type, MNTTYPE_EXT2)) {
               ext2_direct_scan(mnt->mnt_fsname);
-           } else {
+           } else if (mnt->mnt_dir) {
 #else
            if (mnt->mnt_dir) {
 #endif
@@ -635,27 +641,31 @@ void dump_to_file(char *fsname, char *quotafile, int type)
    }
 
    if (fstat(fd, &st) == 0) {
-      max_id = st.st_size / sizeof(struct dqblk);
+      max_id = (st.st_size / sizeof(struct dqblk)) - 1;
       if (max_id < highestid[type])
          max_id = highestid[type];
    } else
       max_id = highestid[type];
 
-   for (id = 1; id <= max_id; id++) {
+   for (id = 0; id <= max_id; id++) {
       memset((caddr_t)&dq_dqb, 0, sizeof(struct dqblk));
+
       if (lseek(fd, dqoff(id), SEEK_SET) == dqoff(id))
          read(fd, &dq_dqb, sizeof(struct dqblk));
+
       if ((dquot = lookup_dquot(id, type)) != NODQUOT) {
          dq_curinodes = dquot->dq_curinodes;
          dq_curblocks = dquot->dq_curblocks;
          if (dflag)
-            fprintf(stderr, "%s %d: curinodes: %d curblocks: %d\n",
+            fprintf(stderr, "%s %d: curinodes: %d curblocks: %d without hardlinks\n",
                     quotatypes[type], id, dq_curinodes, dq_curblocks);
          add_dlinks((__u32 *)&dq_curinodes, (__u32 *)&dq_curblocks, id, type);
          if (dflag)
-            fprintf(stderr, "%s %d: curinodes: %d curblocks: %d\n",
+            fprintf(stderr, "%s %d: curinodes: %d curblocks: %d with hardlinks\n",
                     quotatypes[type], id, dq_curinodes, dq_curblocks);
       } else {
+         if (dq_curinodes == 0 && dq_curblocks == 0)
+            continue;
          dq_curinodes = 0;
          dq_curblocks = 0;
       }
@@ -664,10 +674,9 @@ void dump_to_file(char *fsname, char *quotafile, int type)
        * If the quota is updated with the systemcall it isn't needed to update
        * it in the file. Because the kernel will do that with the next sync.
        */
-      if (quota_enabled) {
+      if (quota_enabled)
          if (quotactl(QCMD(Q_SETUSE, type), fsname, id, (caddr_t)&dq_dqb) == 0)
             continue;
-      }
 
       if (lseek(fd, dqoff(id), SEEK_SET) == dqoff(id))
          write(fd, &dq_dqb, sizeof(struct dqblk));
